#! /usr/bin/env python


import numpy as np


def get_cps_elf(fn_common, save=True):
    """ Get the value of the ELF at the critical points.
    
    Parameters
    ----------
    fn_common : str
             Common string in the filenames containing information about equivalent and inequivalent saddle points.
    save : bool, optional
             If True (default), critical points and corresponding ELF values are saved to a file.
    """
    crit_eq = np.loadtxt(fn_common + "-saddle-eq.dat")
    crit_all = np.loadtxt(fn_common + "-bond.dat")

    # Write a matrix with all bcps and its ELF value
    crit_all_elf = np.zeros((len(crit_all), 4))
    for i, typ in enumerate(crit_eq[:, 0]):
        idx = np.where(crit_all[:, 1] == typ)[0]
        crit_all_elf[idx, 0] = crit_all[idx, 2]
        crit_all_elf[idx, 1] = crit_all[idx, 3]
        crit_all_elf[idx, 2] = crit_all[idx, 4]
        crit_all_elf[idx, 3] = crit_eq[i, -1]

    if save:
        np.savetxt(fn_common + "-saddle.dat", crit_all_elf)

    return crit_all, crit_all_elf


def get_connectivity(elf_val, crit_attr, cps_elf, crit_bond):
    """ Get connectivity matrix at ELF isovalue elf_val.
    
    Parameters
    ----------
    elf_val : float
           Isovalue of the ELF.
    crit_attr : np.array
           Array of attractor positions and indicators, of shape (M, 5).
    cps_elf : np.array
           Array of ELF saddle points and corresponding ELF value, of shape (L, 4).
    crit_bond: np.array
           Array of ELF saddle point indicators (equivalent and non-equivalent), positions, and connected attractors, of shape (L, 7).
    """

    attr_pos = crit_attr[:, 2:5]
    attr_ind = crit_attr[:, 0]

    # CPS that have ELF value elf_val
    idx = np.where(cps_elf[:, -1] == elf_val)[0]
    bond_ind = crit_bond[idx, 0]
    bond_connect = crit_bond[idx, -2:]

    # Number of attractors and  bcps with elf_val
    n_at = len(crit_attr)
    n_crit = len(bond_ind)

    # Connectivity matrix of hydrogens and cps for elf_val
    connect_at_bcp = np.zeros((n_at, n_crit))

    # First check the bcps connected directly to a hydrogen
    for i in range(n_at):
        # Number indicator of attractor in critic2 output
        at_ind = attr_ind[i]
        # The bcps that begin or end at a particular attr (at_ind)
        idx_init = np.where(bond_connect[:, -2] == at_ind)[0]
        idx_end = np.where(bond_connect[:, -1] == at_ind)[0]
        idx_init_end = np.unique(np.concatenate((idx_init, idx_end)))
        connect_at_bcp[i, idx_init_end] = 1

    # Connectivity between attrs through bcps
    connect = np.zeros((n_at, n_at))
    for i in range(n_at):
        for j in range(n_crit):
            for k in range(i):
                if connect_at_bcp[i, j] == 1 and connect_at_bcp[k, j] == 1:
                    connect[i, k] = 1
                    connect[k, i] = 1

    return connect
