#! /usr/bin/env python

import numpy as np
import logging
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D


def get_segment(pt_A, pt_B, N_pts):
    """Gives N_pts points along the segment connecting pt_A and pt_B.

	Parameters
	----------
	pt_A : np.array
	    Starting point of segment, of shape (3,).
	pt_B : np.array
	    Ending point of segment, of shape (3,).
	N_pts : int
	    Number of point to be taken in the segment.
    """
    dist_ab = np.linalg.norm(pt_B - pt_A)
    rab_unit = (pt_B - pt_A) / dist_ab
    segment = np.array([pt_A + i * (dist_ab / (N_pts + 1)) * rab_unit for i in range(1, N_pts + 1)])
    return segment


def plot_connect(fname, atcoords, connectivity, N_pts=20):
    """Plots atcoords and connecting segments according to connectivity (matrix), and saves to fname.

	Parameters
	----------
	fname : str
	    Filename where plot will be saved.
        atcoords : np.array
            Atomic coordinates, of shape (M, 3).
	connectivity : np.array
	    Connectivity matrix, of shape (M, M).
	N_pts : int, optional
	    Number of point to be taken in the segment.
        """

    n_at = len(atcoords)
    fig = plt.figure()
    ax = fig.add_subplot(1, 1, 1, projection="3d")
    ax.scatter(atcoords[:, 0], atcoords[:, 1], atcoords[:, 2], s=50)

    for i in range(n_at):
        for j in range(n_at):
            if connectivity[i, j] == 1:
                seg = get_segment(atcoords[i], atcoords[j], N_pts)
                ax.scatter(seg[:, 0], seg[:, 1], seg[:, 2], s=10)
    plt.savefig(fname)
    plt.close()


def get_separate(connect):
    """Get list of connectivity matrices of separate disconnected clusters.

	Parameters
	----------
        connect : np.array
               Connectivity matrix, of shape (M, M).
        """
    n_at = len(connect)
    separate_gr = []
    new_group = True
    for i in range(n_at):
        idx_gr_i = [
            [m, n]
            for m, listm in enumerate(separate_gr)
            for n, vert in enumerate(listm)
            if vert == i
        ]
        if len(idx_gr_i) == 0:
            new_group = True
            grouped = [i]
        else:
            new_group = False
            grouped = separate_gr[idx_gr_i[0][0]]
        for j in range(n_at):
            if connect[i, j] == 1 or connect[j, i] == 1:
                if j not in grouped:
                    grouped.append(j)
        if new_group:
            separate_gr.append(grouped)
        else:
            separate_gr[idx_gr_i[0][0]] = grouped

    ineq_grp = separate_gr[:]
    for i, gr_lst1 in enumerate(separate_gr):
        for j, gr_lst2 in enumerate(separate_gr):
            if i != j:
                if len(set(gr_lst1 + gr_lst2)) == len(gr_lst1):
                    try:
                        ineq_grp.remove(gr_lst2)
                    except:
                        pass
                elif len(set(gr_lst1 + gr_lst2)) == len(gr_lst2):
                    try:
                        ineq_grp.remove(gr_lst1)
                    except:
                        pass
    ineq_grp = [grp for grp in ineq_grp if len(grp) > 1]
    sep_connect = []
    for grp in ineq_grp:
        connect_grp = np.zeros((len(grp), len(grp)))
        for i, idx_at1 in enumerate(grp):
            for j, idx_at2 in enumerate(grp):
                if connect[idx_at1, idx_at2] == 1:
                    connect_grp[i, j] = 1
        sep_connect.append(connect_grp)

    return ineq_grp, sep_connect


def is_3D(atcoords, connect, thr=1e-10):
    """Distiguishes if network is 3D.

	Parameters
	----------
        atcoords : np.array
               Atomic coordinates, of shape (M, 3)
        connect : np.array
               Connectivity matrix of the system, of shape (M, M)
        thr : float, optional
               Threshold for deviations from a plane, to consider something 3D.
	"""
    n_at = len(atcoords)
    i_candidate = None
    for i in range(n_at):
        j_candidates = []
        for j in range(n_at):
            if connect[i, j] == 1:
                j_candidates.append(j)
            if len(j_candidates) >= 2:
                i_candidate = i
                break
        if i_candidate is not None:
            break

    if i_candidate is None:
        return False

    else:
        # We want to choose a plane formed by two vertices of the network
        vec1 = (atcoords[j_candidates[0]] - atcoords[i_candidate]) / np.linalg.norm(
            atcoords[j_candidates[0]] - atcoords[i_candidate]
        )
        vec2 = (atcoords[j_candidates[1]] - atcoords[i_candidate]) / np.linalg.norm(
            atcoords[j_candidates[1]] - atcoords[i_candidate]
        )
        # The vector perpendicular to that plane:
        vec_ppd = np.cross(vec1, vec2)
        vec_ppd /= np.linalg.norm(vec_ppd)
        if all(
            [
                np.dot(vec_ppd, atcoords[i] - atcoords[j]) < thr
                for i in range(n_at)
                for j in range(i)
            ]
        ):
            return False
        else:
            return True
